"""This module contains WFC3/UVIS1 commissioning tests.
Adapted from ``astrolib/pysynphot/from_commissioning/wfc3_uvis1/test*.py`` .
"""

# STDLIB
import os
import shutil

# THIRD-PARTY
from astropy.utils.data import get_pkg_data_filename

# LOCAL
from ..utils import CommCase

# Local test data
datafiles = ['earthshine.fits', 'el1215a.fits', 'el1302a.fits', 'el1356a.fits',
             'el2471a.fits', 'Zodi.fits']


def setup_module(module):
    """Copy test data to working directory so ``parse_spec`` works
    properly for both ``stsynphot`` and ASTROLIB PYSYNPHOT."""
    for datafile in datafiles:
        src = get_pkg_data_filename(os.path.join('data', datafile))
        shutil.copyfile(src, datafile)


def teardown_module(module):
    """Clean up test data in working directory."""
    for datafile in datafiles:
        os.remove(datafile)


class Test1533(CommCase):
    obsmode = 'wfc3,uvis1,f390m'
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1534(CommCase):
    obsmode = 'wfc3,uvis1,f390w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1535(Test1534):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1536(Test1534):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1537(CommCase):
    obsmode = 'wfc3,uvis1,f395n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1538(Test1537):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1539(Test1537):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1540(CommCase):
    obsmode = 'wfc3,uvis1,f410m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1541(Test1540):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1542(Test1540):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1543(CommCase):
    obsmode = 'wfc3,uvis1,f438w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1544(Test1543):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1545(Test1543):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1546(CommCase):
    obsmode = 'wfc3,uvis1,f467m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1547(Test1546):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1548(Test1546):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1549(CommCase):
    obsmode = 'wfc3,uvis1,f469n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1550(Test1549):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1551(Test1549):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1552(CommCase):
    obsmode = 'wfc3,uvis1,f475w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1553(Test1552):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1554(Test1552):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1555(CommCase):
    obsmode = 'wfc3,uvis1,f475x'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1556(Test1555):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1557(Test1555):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1558(CommCase):
    obsmode = 'wfc3,uvis1,f487n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1559(Test1558):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1560(Test1558):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1561(CommCase):
    obsmode = 'wfc3,uvis1,f502n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1562(Test1561):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


# ASTROLIB PYSYNPHOT did not require extrapolation because its flat spectrum
# has pre-define waveset using default waveset but not stsynphot.
class Test1563(Test1561):
    spectrum = ('rn(unit(1.0,flam),band(sdss,r),28.0,vegamag)+'
                'em(5007.0,5.0,1.0E-13,flam)')
    force = 'extrap'


class Test1564(Test1561):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1565(CommCase):
    obsmode = 'wfc3,uvis1,f547m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1566(Test1565):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1567(Test1565):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1568(CommCase):
    obsmode = 'wfc3,uvis1,f555w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1569(Test1568):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1570(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/agk_81d266_stis_001.fits),'
                '0.05),band(johnson,b),28.0,vegamag)')
    force = 'extrap'


class Test1571(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/alpha_lyr_stis_003.fits),'
                '0.15),band(johnson,b),28.0,vegamag)')


class Test1572(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/bd_28d4211_stis_001.fits),'
                '0.1),band(johnson,b),28.0,vegamag)')


class Test1573(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/bd_75d325_stis_001.fits),'
                '0.15),band(johnson,b),28.0,vegamag)')


class Test1574(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/feige110_stis_001.fits),0.25),'
                'band(johnson,b),28.0,vegamag)')


class Test1575(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/feige34_stis_001.fits),0.2),'
                'band(johnson,b),28.0,vegamag)')


class Test1576(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/g191b2b_mod_004.fits),0.25),'
                'band(johnson,b),28.0,vegamag)')


class Test1577(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/g93_48_004.fits),0.3),'
                'band(johnson,b),28.0,vegamag)')


class Test1578(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/gd108_005.fits),0.1),'
                'band(johnson,b),28.0,vegamag)')
    force = 'extrap'


class Test1579(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/gd153_mod_004.fits),0.15),'
                'band(johnson,b),28.0,vegamag)')


class Test1580(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/gd50_004.fits),0.3),'
                'band(johnson,b),28.0,vegamag)')


class Test1581(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/gd71_mod_005.fits),0.05),'
                'band(johnson,b),28.0,vegamag)')


class Test1582(Test1568):
    spectrum = ('rn(z(spec($PYSYN_CDBS/calspec/grw_70d5824_stis_001.fits),'
                '0.2),band(johnson,b),28.0,vegamag)')


class Test1633(CommCase):
    obsmode = 'wfc3,uvis1,f625w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1634(Test1633):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1635(CommCase):
    obsmode = 'wfc3,uvis1,f631n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1636(Test1635):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1637(Test1635):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1638(CommCase):
    obsmode = 'wfc3,uvis1,f645n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1639(Test1638):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1640(Test1638):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1641(CommCase):
    obsmode = 'wfc3,uvis1,f656n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1642(Test1641):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1643(Test1641):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1644(CommCase):
    obsmode = 'wfc3,uvis1,f657n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1645(Test1644):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1646(Test1644):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1647(CommCase):
    obsmode = 'wfc3,uvis1,f658n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1648(Test1647):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1649(Test1647):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1650(CommCase):
    obsmode = 'wfc3,uvis1,f665n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1651(Test1650):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1652(Test1650):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1653(CommCase):
    obsmode = 'wfc3,uvis1,f673n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1654(Test1653):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1655(Test1653):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1656(CommCase):
    obsmode = 'wfc3,uvis1,f680n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1657(Test1656):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1658(Test1656):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1659(CommCase):
    obsmode = 'wfc3,uvis1,f689m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1660(Test1659):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1661(Test1659):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1662(CommCase):
    obsmode = 'wfc3,uvis1,f763m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1663(Test1662):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1664(Test1662):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1665(CommCase):
    obsmode = 'wfc3,uvis1,f775w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1666(Test1665):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1667(Test1665):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1668(CommCase):
    obsmode = 'wfc3,uvis1,f814w'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1669(Test1668):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1670(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_1.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1671(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_1.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1672(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_10.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


class Test1673(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_10.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1674(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_10.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1675(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_100.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1676(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_100.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1677(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_100.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1678(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_100.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


class Test1679(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_11.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1680(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_11.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


class Test1681(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_114.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1682(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_117.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1683(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_118.fits)'
                ',band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1684(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_12.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1685(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_12.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1686(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_13.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1687(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_14.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1688(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_14.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1689(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_15.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1690(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_16.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1691(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_16.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


class Test1692(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_17.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1693(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_17.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1694(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_18.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1695(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_18.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1696(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_19.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1697(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_19.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


class Test1698(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_2.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1699(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_2.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


class Test1700(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_20.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1701(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_20.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1702(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_22.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1703(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_23.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1704(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_24.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1705(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_25.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1706(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_26.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1707(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_27.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1708(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_29.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1709(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_3.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1710(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_31.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


class Test1711(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_33.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1712(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_34.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1713(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_36.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1714(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_37.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1715(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_38.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1716(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_4.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


class Test1717(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_40.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1718(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_5.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1719(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_5.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1720(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_50.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1721(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_51.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1722(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_52.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


class Test1723(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_53.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1724(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_54.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1725(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_55.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1726(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_56.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


class Test1727(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_6.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1728(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_60.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1729(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_63.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1730(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_63.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1731(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_65.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1732(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_65.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


class Test1733(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_65.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


# Original test used smc but it is no longer supported, so we use smcbar
class Test1734(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_67.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.16,smcbar)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1735(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_67.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used xgal but it is no longer supported, so we use xgalsb
class Test1736(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_69.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.24,xgalsb)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1737(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_76.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.08,gal3)')


# Original test used lmc but it is no longer supported, so we use lmcavg
class Test1738(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_87.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.2,lmcavg)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1739(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_9.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


# Original test used gal1 but it is no longer supported, so we use gal3
class Test1740(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_93.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.04,gal3)')


class Test1741(Test1668):
    spectrum = ('rn(spec($PYSYN_CDBS/grid/pickles/dat_uvk/pickles_uk_95.fits),'
                'band(cousins,i),28.0,vegamag)*ebmvx(0.12,gal3)')


class Test1742(Test1668):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1743(CommCase):
    obsmode = 'wfc3,uvis1,f845m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1744(Test1743):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1745(Test1743):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1746(CommCase):
    obsmode = 'wfc3,uvis1,f850lp'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1747(Test1746):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1748(Test1746):
    spectrum = 'rn(pl(4000.0,-2.0,flam),band(Bessell,j),28.0,vegamag)'


class Test1749(Test1746):
    spectrum = 'rn(pl(4000.0,-2.0,flam),band(bessell,h),28.0,vegamag)'


class Test1750(Test1746):
    spectrum = 'rn(pl(4000.0,-2.0,flam),band(bessell,k),28.0,vegamag)'


class Test1751(Test1746):
    spectrum = ('rn(spec(Zodi.fits),band(johnson,v),22.7,vegamag)+'
                '(spec(el1215a.fits)+spec(el1302a.fits)+spec(el1356a.fits)+'
                'spec(el2471a.fits))')


class Test1752(Test1746):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1753(Test1746):
    spectrum = ('spec(earthshine.fits)*2.0+rn(spec(Zodi.fits),band(johnson,v),'
                '22.7,vegamag)+(spec(el1215a.fits)+spec(el1302a.fits)+'
                'spec(el1356a.fits)+spec(el2471a.fits))')


class Test1754(Test1746):
    spectrum = ('spec(earthshine.fits)+rn(spec(Zodi.fits),band(johnson,v),'
                '22.7,vegamag)+(spec(el1215a.fits)+spec(el1302a.fits)+'
                'spec(el1356a.fits)+spec(el2471a.fits))')


class Test1755(CommCase):
    obsmode = 'wfc3,uvis1,f953n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1756(Test1755):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1757(Test1755):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1758(CommCase):
    obsmode = 'wfc3,uvis1,fq232n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1759(Test1758):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1760(Test1758):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1761(CommCase):
    obsmode = 'wfc3,uvis1,fq243n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1762(Test1761):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1763(Test1761):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1764(CommCase):
    obsmode = 'wfc3,uvis1,fq378n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1765(Test1764):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1766(Test1764):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1767(CommCase):
    obsmode = 'wfc3,uvis1,fq387n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1768(Test1767):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1769(Test1767):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1770(CommCase):
    obsmode = 'wfc3,uvis1,fq422m'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1771(Test1770):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1772(Test1770):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1773(CommCase):
    obsmode = 'wfc3,uvis1,fq436n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1774(Test1773):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1775(Test1773):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1776(CommCase):
    obsmode = 'wfc3,uvis1,fq437n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1777(Test1776):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1778(Test1776):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1779(CommCase):
    obsmode = 'wfc3,uvis1,fq492n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'


class Test1780(Test1779):
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),28.0,vegamag)'


class Test1781(Test1779):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test1782(CommCase):
    obsmode = 'wfc3,uvis1,fq508n'
    spectrum = 'rn(icat(k93models,9230,0.0,4.1),band(johnson,v),22.0,vegamag)'



# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""Test io.py module.

.. note::

    ``read_*()`` functions are tested in other modules where they are used.

"""

# STDLIB
import os
import sys
import warnings

# THIRD-PARTY
import numpy as np
import pytest

# ASTROPY
from astropy.utils.data import get_pkg_data_filename
from astropy.utils.exceptions import AstropyUserWarning

# SYNPHOT
from synphot import exceptions as synexceptions

# LOCAL
from .. import stio
from ..config import conf

if stio.ASTROPY_LT_4_3:
    from astropy.utils.data import _find_pkg_data_path as get_pkg_data_path
else:
    from astropy.utils.data import get_pkg_data_path


class TestIRAFConvert:
    """Test IRAF filename conversions."""
    def setup_class(self):
        self.is_win = sys.platform.startswith('win')
        if self.is_win:
            os.environ['MYTESTPATH'] = 'D:\\path1\\path2\\'
        else:
            os.environ['MYTESTPATH'] = '/path1/path2/'

    @pytest.mark.parametrize('in_str', ('mypath/image.fits',
                                        'C:\\mypath\\image.fits'))
    def test_irafconvert_noop(self, in_str):
        assert stio.irafconvert(in_str) == in_str

    def test_irafconvert_mytestpath(self):
        if self.is_win:
            in_str = '$MYTESTPATH\\\\image.fits'
            ans = 'D:\\path1\\path2\\image.fits'
        else:
            in_str = '$MYTESTPATH//image.fits'
            ans = '/path1/path2/image.fits'

        assert stio.irafconvert(in_str) == ans

    @pytest.mark.parametrize(
        ('in_str', 'args'),
        [('CRREFER$image.fits', ('image.fits', )),
         ('mtab$image.fits', ('mtab', 'image.fits'))])
    def test_irafconvert(self, in_str, args):
        ans = stio.resolve_filename(conf.rootdir, *args)
        assert stio.irafconvert(in_str) == ans

    def test_irafconvert_data(self):
        out_str = stio.irafconvert('synphot$detectors.dat')
        assert out_str.endswith(os.path.join('data', 'detectors.dat'))

    def test_exceptions(self):
        with pytest.raises(TypeError):
            stio.irafconvert(1)
        with pytest.raises(KeyError):
            stio.irafconvert('dummydummy$image.fits')

    def teardown_class(self):
        del os.environ['MYTESTPATH']


class TestGetLatestFile:
    """Test getting latest file."""
    def setup_class(self):
        self.datadir = get_pkg_data_path('data')

    @pytest.mark.remote_data
    def test_http(self):
        """Remote HTTP path."""
        path = 'https://ssb.stsci.edu/trds/mtab/OLD_FILES/'
        with warnings.catch_warnings():
            # Warning issued from html5lib 1.0.1
            warnings.filterwarnings(
                'ignore', message=r'.*Using or importing the ABC.*',
                category=DeprecationWarning)
            filename = stio.get_latest_file(
                path + 'n*tmg.fits', raise_error=True)
        assert filename == path + 'n9i1408hm_tmg.fits'

    def test_local(self):
        """Local data path."""
        template = os.path.join(self.datadir, '*tmg.fits')
        ans = os.path.join(self.datadir, 'tables_tmg.fits')
        filename = stio.get_latest_file(template, raise_error=True)
        assert filename == ans

    def test_local_curdir(self):
        curdir = os.getcwd()
        try:
            os.chdir(self.datadir)
            ans = 'tables_tmg.fits'
            assert (stio.get_latest_file(ans, raise_error=True) ==
                    os.path.join('.', ans))
        finally:
            os.chdir(curdir)

    def test_bogus(self):
        """Bogus data path."""
        with pytest.warns(AstropyUserWarning, match=r'No files found'):
            filename = stio.get_latest_file(
                os.path.join('foo', 'foo', '*tmg.fits'))
        assert filename == ''

    def test_no_files(self):
        """Real path with no files."""
        template = os.path.join(self.datadir, '*dummy')

        # Warning only
        with pytest.warns(AstropyUserWarning, match=r'No files found'):
            filename = stio.get_latest_file(template)
        assert filename == ''

        # Raise error
        with pytest.raises(IOError):
            filename = stio.get_latest_file(template, raise_error=True)


def test_read_table_exception():
    """Test data type check for FITS table read."""
    tabname = get_pkg_data_filename(os.path.join('data', 'tables_tmg.fits'))

    # Set INNODE to str (should be int)
    with pytest.raises(synexceptions.SynphotError):
        stio._read_table(
            tabname, 1,
            {'COMPNAME': np.str_, 'KEYWORD': np.str_, 'INNODE': np.str_,
             'OUTNODE': np.int64, 'THCOMPNAME': np.str_, 'COMMENT': np.str_})



"""This module contains ACS/HRC commissioning tests.
Adapted from ``astrolib/pysynphot/from_commissioning/acs/test1.py``,
``astrolib/pysynphot/from_commissioning/acs/test2.py``, and most of
``astrolib/pysynphot/from_commissioning/acs/test3.py``.
"""

# STDLIB
import os
import shutil

# THIRD-PARTY
from astropy.utils.data import get_pkg_data_filename

# LOCAL
from ..utils import CommCase

# Local test data
datafiles = ['earthshine.fits', 'el1215a.fits', 'el1302a.fits', 'el1356a.fits',
             'el2471a.fits', 'Zodi.fits']


def setup_module(module):
    """Copy test data to working directory so ``parse_spec`` works
    properly for both ``stsynphot`` and ASTROLIB PYSYNPHOT."""
    for datafile in datafiles:
        src = get_pkg_data_filename(os.path.join('data', datafile))
        shutil.copyfile(src, datafile)


def teardown_module(module):
    """Clean up test data in working directory."""
    for datafile in datafiles:
        os.remove(datafile)


class Test472(CommCase):
    obsmode = 'acs,hrc,coron,fr388n#3880'
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test473(Test472):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),5,vegamag)'


class Test474(Test472):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test475(Test472):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),30.0,vegamag)')


class Test476(Test472):
    spectrum = 'spec(earthshine.fits)*0.5+spec(Zodi.fits)*1.0'


class Test477(Test472):
    spectrum = 'spec(earthshine.fits)*0.5+spec(Zodi.fits)*1.25'


class Test478(Test472):
    spectrum = 'spec(earthshine.fits)*0.5+spec(Zodi.fits)*2.0'


class Test479(Test472):
    spectrum = 'spec(earthshine.fits)*0.5+spec(Zodi.fits)*4.0'


class Test480(CommCase):
    obsmode = 'acs,hrc,f220w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test481(Test480):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test482(Test480):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


# Duplicate of Test480 and the syntax already tested in Test507
# class Test483(Test480):
#    spectrum = 'crcalspec$g191b2b_mod_004.fits'


class Test484(Test480):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test485(Test480):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test486(CommCase):
    obsmode = 'acs,hrc,f250w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test487(Test486):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


# Duplicate of Test490 and syntax already tested in Test482
# class Test488(Test486):
#    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test489(Test486):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test490(Test486):
    spectrum = 'spec($PYSYN_CDBS/calspec/gd71_mod_005.fits)'


class Test491(Test486):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test492(CommCase):
    obsmode = 'acs,hrc,f330w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test493(Test492):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test494(Test492):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test495(Test492):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.0e-17,flam)'


class Test496(Test492):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


# Duplicate of Test494 and syntax already tested in Test490
# class Test497(Test492):
#    spectrum = 'spec($PYSYN_CDBS/calspec/gd71_mod_005.fits)'


class Test498(Test492):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test499(CommCase):
    obsmode = 'acs,hrc,f344n'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test500(Test499):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test501(Test499):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test502(Test499):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test503(Test499):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


# Duplicate of Test507 and syntax already tested in Test499
# class Test504(CommCase):
#    obsmode = 'acs,hrc,f435w'
#    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test505(CommCase):
    obsmode = 'acs,hrc,f435w'
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test506(Test505):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test507(Test505):
    spectrum = 'crcalspec$g191b2b_mod_004.fits'


class Test508(Test505):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test509(Test505):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test510(CommCase):
    obsmode = 'acs,hrc,f475w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test511(Test510):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test512(Test510):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test513(Test510):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test514(Test510):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test515(CommCase):
    obsmode = 'acs,hrc,f502n'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test516(Test515):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test517(Test515):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test518(Test515):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test519(Test515):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test520(CommCase):
    obsmode = 'acs,hrc,f550m'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test521(Test520):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test522(Test520):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test523(Test520):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test524(Test520):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test525(CommCase):
    obsmode = 'acs,hrc,f555w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test526(Test525):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test527(Test525):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test528(Test525):
    spectrum = 'rn(bb(10000),band(johnson,v),20,vegamag)'


class Test529(Test525):
    spectrum = 'rn(icat(k93models,5770,0.0,4.5),band(johnson,v),20,vegamag)'


class Test530(Test525):
    spectrum = 'rn(pl(4000.0,-1.0,flam),band(johnson,v),20,vegamag)'


class Test531(Test525):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test532(Test525):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test533(Test525):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test534(Test525):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test535(CommCase):
    obsmode = 'acs,hrc,f555w,coron'
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),0,vegamag)'


class Test536(Test535):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),10,vegamag)'


class Test537(Test535):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test538(Test535):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),5,vegamag)'


class Test539(Test535):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test540(CommCase):
    obsmode = 'acs,hrc,f606w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test541(Test540):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test542(Test540):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test543(Test540):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test544(Test540):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test545(CommCase):
    obsmode = 'acs,hrc,f625w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test546(Test545):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test547(Test545):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test548(Test545):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test549(Test545):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test550(CommCase):
    obsmode = 'acs,hrc,f658n'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test551(Test550):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test552(Test550):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test553(Test550):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test554(Test550):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test555(CommCase):
    obsmode = 'acs,hrc,f660n'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test556(Test555):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test557(Test555):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test558(CommCase):
    obsmode = 'acs,hrc,f775w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test559(Test558):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test560(Test558):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


# Duplicate of Test558 and syntax already tested in Test507.
# class Test561(Test558):
#    spectrum = 'crcalspec$g191b2b_mod_004.fits'


class Test562(Test558):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test563(Test558):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test564(CommCase):
    obsmode = 'acs,hrc,f814w'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test565(Test564):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test566(Test564):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


# Duplicate of Test564 and syntax already tested in Test507.
# class Test567(Test564):
#    spectrum = 'crcalspec$g191b2b_mod_004.fits'


class Test568(CommCase):
    obsmode = 'acs,hrc,f850lp'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test569(Test568):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test570(Test568):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test571(Test568):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test572(Test568):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test573(CommCase):
    obsmode = 'acs,hrc,f892n'
    spectrum = '$PYSYN_CDBS/calspec/g191b2b_mod_004.fits'


class Test574(Test573):
    spectrum = '$PYSYN_CDBS/calspec/gd153_mod_004.fits'


class Test575(Test573):
    spectrum = '$PYSYN_CDBS/calspec/gd71_mod_005.fits'


class Test576(Test573):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1e-18,flam)'


class Test577(Test573):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test578(CommCase):
    obsmode = 'acs,hrc,fr388n#3880'
    spectrum = 'rn(bb(10000),band(johnson,v),20,vegamag)'


class Test579(Test578):
    spectrum = 'rn(icat(k93models,15400,0.0,3.9),band(johnson,v),15,vegamag)'


class Test580(Test578):
    spectrum = 'rn(icat(k93models,3500,0.0,4.6),band(johnson,v),15,vegamag)'


class Test581(Test578):
    spectrum = 'rn(icat(k93models,44500,0.0,5.0),band(johnson,v),15,vegamag)'


class Test582(Test578):
    spectrum = 'rn(icat(k93models,4850,0.0,1.1),band(johnson,v),15,vegamag)'


class Test583(Test578):
    spectrum = 'rn(icat(k93models,5770,0.0,4.5),band(johnson,v),15,vegamag)'


class Test584(Test578):
    spectrum = 'rn(icat(k93models,5770,0.0,4.5),band(johnson,v),20,vegamag)'


class Test585(Test578):
    spectrum = 'rn(pl(4000.0,-1.0,flam),band(johnson,v),20,vegamag)'


class Test586(Test578):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test587(Test578):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.E-15,flam)'


# Duplicate of Test587 and syntax already tested in Test571
# class Test588(Test578):
#    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.e-15,flam)'


class Test589(Test578):
    spectrum = 'spec($PYSYN_CDBS/calspec/g191b2b_mod_004.fits)'


class Test590(Test578):
    spectrum = 'spec($PYSYN_CDBS/calspec/gd71_mod_005.fits)'


class Test591(Test578):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test592(CommCase):
    obsmode = 'acs,hrc,fr459m#4590'
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),22,vegamag)'


class Test593(Test592):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.e-15,flam)'


class Test594(Test592):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test595(CommCase):
    obsmode = 'acs,hrc,fr459m#4592'
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test596(Test595):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test597(CommCase):
    obsmode = 'acs,hrc,fr505n#5050'
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.e-15,flam)'


class Test598(Test597):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test599(CommCase):
    obsmode = 'acs,hrc,fr656n#6560'
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.e-15,flam)'


class Test600(Test599):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test601(CommCase):
    obsmode = 'acs,hrc,g800l'
    spectrum = 'em(6500.0,10.0,1.0E-16,flam)'


class Test602(Test601):
    spectrum = 'rn(bb(10000),band(johnson,v),20,vegamag)'


class Test603(Test601):
    spectrum = 'rn(icat(k93models,5770,0.0,4.5),band(johnson,v),20,vegamag)'


class Test604(Test601):
    spectrum = 'rn(pl(4000.0,-1.0,flam),band(johnson,v),20,vegamag)'


class Test605(Test601):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test606(Test601):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.5e-16,flam)'


class Test607(Test601):
    spectrum = 'spec($PYSYN_CDBS/calspec/gd71_mod_005.fits)'


class Test608(Test601):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test609(Test601):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')


class Test610(CommCase):
    obsmode = 'acs,hrc,pr200l'
    spectrum = 'em(4000.0,10.0,1.0E-16,flam)'


class Test611(Test610):
    spectrum = 'rn(bb(10000),band(johnson,v),20,vegamag)'


class Test612(Test610):
    spectrum = 'rn(icat(k93models,5770,0.0,4.5),band(johnson,v),20,vegamag)'


class Test613(Test610):
    spectrum = 'rn(pl(4000.0,-1.0,flam),band(johnson,v),20,vegamag)'


class Test614(Test610):
    spectrum = 'rn(unit(1.0,flam),band(johnson,v),15,vegamag)'


class Test615(Test610):
    spectrum = 'rn(unit(1.0,flam),box(5500.0,1.0),1.5e-16,flam)'


class Test616(Test610):
    spectrum = 'spec($PYSYN_CDBS/calspec/gd71_mod_005.fits)'


class Test617(Test610):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)')


class Test618(Test610):
    spectrum = ('spec(earthshine.fits)*0.5+rn(spec(Zodi.fits),'
                'band(johnson,v),22.7,vegamag)+(spec(el1215a.fits)+'
                'spec(el1302a.fits)+spec(el1356a.fits)+spec(el2471a.fits))')



# Licensed under a 3-clause BSD style license - see LICENSE.rst
"""Module to handle ``stsynphot.config.conf.wavecatfile`` table,
which is used by ETC to select an appropriate wavelength set
for a given observation mode for count rate calculations.

"""

# STDLIB
import warnings

# THIRD-PARTY
import numpy as np

# ASTROPY
from astropy import units as u
from astropy.utils.exceptions import AstropyUserWarning

# SYNPHOT
from synphot import units

# LOCAL
from . import exceptions, stio
from .config import conf

__all__ = ['WAVECAT', 'WaveCatalog', 'load_wavecat']

# This is NSPEC-1 in IRAF SYNPHOT COUNTRATE calcstep.x.
# NSPEC was hardcoded to 2000 as the number of bins into
# which the wavelength set should be divided by default.
_N_SPEC = 1999.0

# Will be initialized by load_wavecat()
WAVECAT = None


class WaveCatalog:
    """Class to handle ``stsynphot.config.conf.wavecatfile`` initialization
    and access.

    Input file is parsed with :func:`stsynphot.stio.read_wavecat`.

    For each observation mode, its wavelength table is defined
    by filename or parameter string. When it is accessed with
    :py:meth:`~object.__getitem__`, the string is replaced by the actual
    wavelengths array. If filename is given, it is parsed with
    :func:`stsynphot.stio.read_waveset`.

    Parameters
    ----------
    fname : str
        Wavecat filename.

    wave_unit : str or `~astropy.units.Unit`
        Wavelength unit.

    Attributes
    ----------
    file : str
        Wavecat filename.

    wave_unit : `~astropy.units.Unit`
        Wavelength unit.

    lookup : dict
        Maps observation mode to corresponding wavelength table
        filename or parameter string.

    setlookup : dict
        Maps individual components of observation mode to its
        string. Used for partial matching.

    """
    def __init__(self, fname, wave_unit=u.AA):
        data = stio.read_wavecat(stio.irafconvert(fname))

        self.file = fname
        self.wave_unit = units.validate_wave_unit(wave_unit)
        self.lookup = {}
        self.setlookup = {}

        for line in data:
            obm = line['OBSMODE']
            coeff = line['FILENAME']
            self.lookup[obm] = coeff
            self.setlookup[frozenset(obm.split(','))] = obm

    def __getitem__(self, key):
        """Fairly smart lookup by observation mode.
        If no exact match, find the most complete match.

        """
        # Find exact match
        try:
            ans = self.lookup[key]
        # Find the next most complete match
        except KeyError:
            ans = None
            # Try a set-wise match.
            # The correct key will be a subset of the input key.
            setkey = set(key.split(','))
            candidates = [k for k in self.setlookup if k.issubset(setkey)]
            n_match = len(candidates)
            # We may have 1, 0, or >1 candidates.
            if n_match == 1:
                ans = self.lookup[self.setlookup[candidates[0]]]
            elif n_match == 0:
                raise KeyError(f'{setkey} not found in {self.file}; '
                               f'candidates: {str(candidates)}')
            elif n_match > 1:
                setlens = np.array([len(k) for k in candidates])
                srtlen = setlens.argsort()
                k, j = srtlen[-2:]
                # It's really ambiguous
                if setlens[k] == setlens[j]:
                    raise exceptions.AmbiguousObsmode(
                        f'{setkey}; candidates: {str(candidates)}')
                # We have a winner
                else:
                    k = candidates[srtlen[-1]]
                    ans = self.lookup[self.setlookup[k]]
        return ans

    @staticmethod
    def _calc_quadratic_coeff(coeff):
        """Calculate quadratic coefficients from parameter string."""
        coefficients = coeff[1:-1].split(',')
        n_coeff = len(coefficients)

        c0 = float(coefficients[0])
        c1 = float(coefficients[1])
        c2 = (c1 - c0) / _N_SPEC
        c3 = c2

        if n_coeff > 2:
            c2 = float(coefficients[2])
            c3 = c2
        if n_coeff > 3:
            c3 = float(coefficients[3])

        nwave = int(2.0 * (c1 - c0) / (c3 + c2)) + 1
        c = c0
        b = c2
        a = (c3 * c3 - c2 * c2) * 0.25 / (c1 - c0)

        return a, b, c, nwave

    def _waveset_from_parstring(self, coeff):
        """Get wavelengths array from parameter string."""
        a, b, c, nwave = self._calc_quadratic_coeff(coeff)
        i = np.arange(nwave, dtype=np.float64)
        return (((a * i) + b) * i + c) * self.wave_unit

    def load_waveset(self, obsmode):
        """Load wavelength table by observation mode.
        If no exact match, find the most complete match.

        Parameters
        ----------
        obsmode : str
            Observation mode.

        Returns
        -------
        par : str
            Matched filename or parameter string.
            May be exact or closest.

        waveset : `astropy.units.quantity.Quantity`
            Corresponding wavelength set.

        """
        par = self.__getitem__(obsmode)
        if par.startswith('('):
            waveset = self._waveset_from_parstring(par)
        else:
            waveset = stio.read_waveset(
                stio.irafconvert(par), wave_unit=self.wave_unit)
        return par, waveset


def load_wavecat(wave_unit=u.AA):
    """Convenience function to update ``stsynphot.wavetable.WAVECAT``
    global variable with the latest ``stsynphot.config.conf.wavecatfile``.

    Parameters
    ----------
    wave_unit : str or `~astropy.units.Unit`
        See `WaveCatalog`.

    """
    global WAVECAT
    WAVECAT = WaveCatalog(conf.wavecatfile, wave_unit=wave_unit)


# Load default wavecat file in Angstrom.
try:
    load_wavecat()
except Exception as e:
    warnings.warn(
        f'Failed to load {conf.wavecatfile}: {repr(e)}', AstropyUserWarning)
